@@ -1,7 +1,7 @@
 # -*- perl -*-
 ##----------------------------------------------------------------------------
 ## REST API Framework - ~/lib/Net/API/REST.pm
-## Version v1.0.0
+## Version v1.0.1
 ## Copyright(c) 2023 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2019/09/01
@@ -42,7 +42,7 @@ BEGIN
     use Net::API::REST::Request;
     use Net::API::REST::Response;
     use Apache2::API::Status;
-    $VERSION = 'v1.0.0';
+    $VERSION = 'v1.0.1';
 };
 
 use strict;
@@ -128,24 +128,35 @@ sub handler : method
     # https://perl.apache.org/docs/2.0/user/handlers/http.html#HTTP_Request_Handler_Skeleton
     my( $class, $r ) = @_;
     # my $handlerClass = $r->dir_config( 'Net_API_REST_Handler' ) || 'Net::API::REST' ;
-    my $debug = $r->dir_config( 'DEBUG' ) || 0;
-    my $req = Net::API::REST::Request->new( $r, debug => $DEBUG );
+    my $debug = $r->dir_config( 'DEBUG' ) || $DEBUG;
+    my $req = Net::API::REST::Request->new( $r, debug => $debug );
     # An error has occurred
     if( !defined( $req ) )
     {
+        $r->log_error( "${class}::handler: Error instantiating an Net::API::REST::Request object: " . Net::API::REST::Request->error );
         return( Net::API::REST::Request->error->code || Apache2::Const::HTTP_INTERNAL_SERVER_ERROR );
     }
-    my $resp = Net::API::REST::Response->new( request => $req, debug => $DEBUG );
+    my $resp = Net::API::REST::Response->new( request => $req, debug => $debug ) || do
+    {
+        $r->log_error( "${class}::handler: Error instantiating an Net::API::REST::Response object: " . Net::API::REST::Response->error );
+        return( Net::API::REST::Response->error->code || Apache2::Const::HTTP_INTERNAL_SERVER_ERROR );
+    };
     my $self = $class->new(
         apache_request => $r,
         debug       => $debug,
         request     => $req,
         response    => $resp,
-    );
+    ) || do
+    {
+        $r->log_error( "Error instantiating a new $class object: ", $class->error );
+        return( Apache2::Const::HTTP_INTERNAL_SERVER_ERROR );
+    };
+    
     if( my $code = $self->log_handler )
     {
         $r->set_handlers( 'PerlPrivateLogHandler' => $code );
     }
+    
     # $self->apache_request( $r );
     # $r->log_error( "Received Apache request $r, object debug value is: ", $self->debug );
     # Full uri. $r->uri only returns the path
@@ -196,7 +207,7 @@ sub handler : method
     # Only the content type, ie without the charset. Example: application/json; charset=utf-8
     my $ct = lc( $req->type );
     # We check if content type is provided at all since it could be missing, such as in 
-    # a POST or GET request with no content or query
+    # a OPTIONS, POST or GET request with no content or query
     if( length( $ct ) && scalar( @$ok_ct ) && !scalar( grep( $ct eq $_, @$ok_ct ) ) )
     {
         # Net::API::REST::reply will automatically set a json with an error message based on the user language
@@ -216,7 +227,7 @@ sub handler : method
                 $client_api_version_is_ok++;
             }
         }
-        return( $self->reply( Apache2::Const::HTTP_NOT_ACCEPTABLE, { error => "API version requested ($client_version) is not supported." } ) );
+        return( $self->reply( Apache2::Const::HTTP_NOT_ACCEPTABLE, { error => "API version requested ($client_version) is not supported." } ) ) unless( $client_api_version_is_ok );
     }
     
     # Protection against DNS rebinding attacks
@@ -229,6 +240,9 @@ sub handler : method
             # Net::API::REST::reply will automatically set a json with an error message based on the user language
             return( $self->reply({ code => Apache2::Const::HTTP_UNAUTHORIZED }) );
         }
+        else
+        {
+        }
     }
     
     $self->request( $req );
@@ -252,7 +266,10 @@ sub handler : method
     }
     
     my $origin = $req->headers( 'Origin' );
-    $self->http_cors if( $origin );
+    if( $origin )
+    {
+        $self->http_cors;
+    }
     
     my $path = $uri->path;
     if( my $base = $r->dir_config( 'Net_API_REST_Base' ) ) 
@@ -387,12 +404,23 @@ Variables .....: %s
 EOT
     $self->noexec->messagef( 3, $tmpl, $ep->handler, $ep->access, join( ', ', @{$ep->methods} ), join( ', ', @{$ep->path_info} ), $self->dumper( $ep->variables->as_hash ) );
 
-    my $req_methods = [CORE::split( /\,[[:blank:]]*/, $req->headers( 'Access-Control-Request-Method' ) )];
-    foreach my $m ( @$req_methods )
+    my $req_methods = [CORE::split( /\,[[:blank:]]*/, ( $req->headers( 'Access-Control-Request-Method' ) || '' ) )];
+    if( scalar( @$req_methods ) )
     {
-        if( !$ep->is_method_allowed( $m ) )
+        foreach my $m ( @$req_methods )
         {
-            return( Apache2::Const::HTTP_METHOD_NOT_ALLOWED );
+            if( !$ep->is_method_allowed( $m ) )
+            {
+                return( Apache2::Const::HTTP_METHOD_NOT_ALLOWED );
+            }
+        }
+        if( !$ep->methods->is_empty )
+        {
+            $res->headers( 'Access-Control-Allow-Methods' => $ep->methods->join( ', ' )->scalar );
+        }
+        else
+        {
+            $res->headers( 'Access-Control-Allow-Methods' => '*' );
         }
     }
     
@@ -1386,11 +1414,11 @@ sub is_method_allowed
     return( scalar( grep( /^$meth$/i, @$ok_methods ) ) );
 }
 
-sub methods { return( shift->_set_get_array( 'methods', @_ ) ); }
+sub methods { return( shift->_set_get_array_as_object( 'methods', @_ ) ); }
 
 sub path { return( shift->_set_get_uri( 'path', @_ ) ); }
 
-sub path_info { return( shift->_set_get_array( 'path_info', @_ ) ); }
+sub path_info { return( shift->_set_get_array_as_object( 'path_info', @_ ) ); }
 
 # sub variables { return( shift->_set_get_hash_as_object( 'variables', 'Net::API::REST::Endpoint::Variables', @_ ) ); }
 sub variables { return( shift->_set_get_hash_as_mix_object( 'variables', @_ ) ); }
@@ -1553,7 +1581,7 @@ Net::API::REST - Framework for RESTful APIs
 
 =head1 VERSION
 
-    v1.0.0
+    v1.0.1
 
 =head1 DESCRIPTION
 
